@@ -69,9 +69,11 @@ intrinsic.operators <- function(C,
       
       if (d > 1) {
         if (compute_higher_order) {
-          fem <- INLA::inla.mesh.fem(mesh, order = m_alpha + 1)
+          # fem <- INLA::inla.mesh.fem(mesh, order = m_alpha + 1)
+          fem <- fmesher::fm_fem(mesh, order = m_alpha + 1)
         } else {
-          fem <- INLA::inla.mesh.fem(mesh)
+          # fem <- INLA::inla.mesh.fem(mesh)
+          fem <- fmesher::fm_fem(mesh)
         }
         
         C <- fem$c0
@@ -85,7 +87,8 @@ intrinsic.operators <- function(C,
           Gk[[i]] <- fem[[paste0("g", i)]]
         }
       } else if (d == 1) {
-        fem <- INLA::inla.mesh.fem(mesh, order = 2)
+        # fem <- INLA::inla.mesh.fem(mesh, order = 2)
+        fem <- fmesher::fm_fem(mesh)
         C <- fem$c0
         G <- fem$g1
         Ci <- Matrix::Diagonal(dim(C)[1], 1 / rowSums(C))
@@ -427,22 +430,24 @@ intrinsic.precision <- function(alpha, rspde.order, dim, fem_mesh_matrices,
 #' conditions and a zero-mean constraint is additionally imposed to obtained 
 #' a non-intrinsic model. 
 #' @examples
-#' x <- seq(from = 0, to = 10, length.out = 201)
-#' beta <- 1
-#' alpha <- 1
-#' kappa <- 1
-#' op <- intrinsic.matern.operators(kappa = kappa, tau = 1, alpha = alpha, 
+#' if (requireNamespace("RSpectra", quietly = TRUE)){
+#'  x <- seq(from = 0, to = 10, length.out = 201)
+#'  beta <- 1
+#'  alpha <- 1
+#'  kappa <- 1
+#'  op <- intrinsic.matern.operators(kappa = kappa, tau = 1, alpha = alpha, 
 #'                                  beta = beta, loc_mesh = x, d=1) 
-#' # Compute and plot the variogram of the model
-#' Sigma <- op$A %*% solve(op$Q,t(op$A))
-#' One <- rep(1, times = ncol(Sigma))
-#' D <- diag(Sigma)
-#' Gamma <- 0.5*(One %*% t(D) + D %*% t(One) - 2 * Sigma)
-#' k <- 100
-#' plot(x, Gamma[k, ], type = "l")
-#' lines(x, 
+#'  # Compute and plot the variogram of the model
+#'  Sigma <- op$A %*% solve(op$Q,t(op$A))
+#'  One <- rep(1, times = ncol(Sigma))
+#'  D <- diag(Sigma)
+#'  Gamma <- 0.5*(One %*% t(D) + D %*% t(One) - 2 * Sigma)
+#'  k <- 100
+#'  plot(x, Gamma[k, ], type = "l")
+#'  lines(x, 
 #'       variogram.intrinsic.spde(x[k], x, kappa, alpha, beta, L = 10, d = 1),
 #'       col=2, lty = 2)
+#' }
 intrinsic.matern.operators <- function(kappa,
                                        tau,
                                        alpha ,
@@ -518,7 +523,8 @@ intrinsic.matern.operators <- function(kappa,
   
   if (!is.null(mesh)) {
     d <- get_inla_mesh_dimension(inla_mesh = mesh)
-    fem <- INLA::inla.mesh.fem(mesh)
+    # fem <- INLA::inla.mesh.fem(mesh)
+    fem <- fmesher::fm_fem(mesh)
     C <- fem$c0
     G <- fem$g1
     has_mesh <- TRUE
@@ -541,11 +547,12 @@ intrinsic.matern.operators <- function(kappa,
   
   if(!is.null(mesh)){
     make_A <- function(loc){
-      return(INLA::inla.spde.make.A(mesh = mesh, loc = loc))
+      # return(INLA::inla.spde.make.A(mesh = mesh, loc = loc))
+      return(fmesher::fm_basis(x = mesh, loc = loc))
     }
   } else if(!is.null(graph)){
     make_A <- function(loc){
-      return(graph$mesh_A(loc))
+      return(graph$fem_basis(loc))
     }
   } else if(!is.null(loc_mesh) && d == 1){
     make_A <- function(loc){
@@ -724,22 +731,24 @@ intrinsic.matern.operators <- function(kappa,
 #' @seealso [intrinsic.matern.operators()]
 #'
 #' @examples
-#' x <- seq(from = 0, to = 10, length.out = 201)
-#' beta <- 1
-#' alpha <- 1
-#' kappa <- 1
-#' op <- intrinsic.matern.operators(kappa = kappa, tau = 1, alpha = alpha, 
+#' if (requireNamespace("RSpectra", quietly = TRUE)){
+#'  x <- seq(from = 0, to = 10, length.out = 201)
+#'  beta <- 1
+#'  alpha <- 1
+#'  kappa <- 1
+#'  op <- intrinsic.matern.operators(kappa = kappa, tau = 1, alpha = alpha, 
 #'                                  beta = beta, loc_mesh = x, d=1) 
-#' # Compute and plot the variogram of the model
-#' Sigma <- op$A %*% solve(op$Q,t(op$A))
-#' One <- rep(1, times = ncol(Sigma))
-#' D <- diag(Sigma)
-#' Gamma <- 0.5*(One %*% t(D) + D %*% t(One) - 2 * Sigma)
-#' k <- 100
-#' plot(x, Gamma[k, ], type = "l")
-#' lines(x, 
+#'  # Compute and plot the variogram of the model
+#'  Sigma <- op$A %*% solve(op$Q,t(op$A))
+#'  One <- rep(1, times = ncol(Sigma))
+#'  D <- diag(Sigma)
+#'  Gamma <- 0.5*(One %*% t(D) + D %*% t(One) - 2 * Sigma)
+#'  k <- 100
+#'  plot(x, Gamma[k, ], type = "l")
+#'  lines(x, 
 #'       variogram.intrinsic.spde(x[k], x, kappa, alpha, beta, L = 10, d = 1),
 #'       col=2, lty = 2)
+#' }
 variogram.intrinsic.spde <- function(s0 = NULL, 
                                      s = NULL, 
                                      kappa = NULL, 
